﻿------------------------------------------------------------------------------
-- $Id$
------------------------------------------------------------------------------
--                            module_combat.lua
--
--    Copyright (C) 2012 Deronda
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU Lesser General Public License as published
--  by the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--  This file is for use with clcInfo by abija.  Some of the code was pulled
--  from existing clcInfo modules and the example module.
--
--  abija has distributed clcInfo All Rights Reserved.  For more information
--  go to: http://wow.curseforge.com/addons/clctracker/
--
------------------------------------------------------------------------------
-- Author:
--      Deronda (@ Stormreaver)
------------------------------------------------------------------------------
-- Description:
--   Actions module for combat rogues
------------------------------------------------------------------------------
if not clcInfo then return end       -- need clcInfo to run
if not clcInfo_Rogue then return end -- need clcInfo_Rogue
local _, englishClass = UnitClass("player") -- need to be a rogue
if englishClass ~= clcInfo_Rogue.CLASS then return end

-- Get local copies of functions/tables.  This will speed up execution.
local clcInfo_Rogue = clcInfo_Rogue
local spells = clcInfo_Rogue.spells
local GetAuraStacks = clcInfo_Rogue.GetAuraStacks
local GetTimeLeft = clcInfo_Rogue.GetTimeLeft
local GetCooldown = clcInfo_Rogue.GetCooldown
local IconNext = clcInfo_Rogue.IconNext
local IconCurrent = clcInfo_Rogue.IconCurrent
local IsStealthed = clcInfo_Rogue.IsStealthed
local BleedDebuffLeftOnTarget = clcInfo_Rogue.BleedDebuffLeftOnTarget
local UnitPower = UnitPower
local UnitAura = UnitAura
local UnitInRaid = UnitInRaid
local UnitInParty = UnitInParty
local IsSpellKnown = IsSpellKnown
clcInfo_Rogue.COMBAT_MAX_NRG_SPECIALS = 50
local MAX_NRG_SPECIALS = clcInfo_Rogue.COMBAT_MAX_NRG_SPECIALS

-- Assign module variables
local specname = "combat"
local prio_arg = "combatprio"
local sample_prio = "prep vanish ambush snd expose sb ks ar rupture eviscerate rs tricks ss"
local priority = {"none","none","ambush","snd","expose","none","none","none",
                  "rupture","eviscerate","rs","none","ss"}
local class_module
local class_module_name

local db_default = {
    char_var = false,
}
local db

local tdb_default = {
    version = clcInfo_Rogue.VERSION,
    priority = priority,
    rangePerSkill = false,
}
local tdb

-- actions --------------------------------------------------------------------
-- local copies of spell names and ids to ugly up code but speed action
-- functions up
local AdrenalineRush = spells["Adrenaline Rush"].name
local BladeFlurry = spells["Blade Flurry"].name
local DeepInsight = spells["Deep Insight"].name
local FuryoftheDestroyer = spells["Fury of the Destroyer"].name
local ModerateInsight =  spells["Moderate Insight"].name
local RevealingStrike = spells["Revealing Strike"].name
local Rupture = spells["Rupture"].name
local ShadowBlades = spells["Shadow Blades"].name
local ShallowInsight = spells["Shallow Insight"].name
local SliceAndDice = spells["Slice and Dice"].name
local WeakenedArmor = spells["Weakened Armor"].name

local AdrenalineRushId = spells["Adrenaline Rush"].id
local AmbushId = spells["Ambush"].id
local AnticipationId = spells["Anticipation"].id
local EviscerateId = spells["Eviscerate"].id
local ExposeId = spells["Expose Armor"].id
local KillingSpreeId = spells["Killing Spree"].id
local PreparationId = spells["Preparation"].id
local RevealingStrikeId = spells["Revealing Strike"].id
local RuptureId = spells["Rupture"].id
local ShadowBladesId = spells["Shadow Blades"].id
local ShadowFocusId = spells["Shadow Focus"].id
local SinisterStrikeId = spells["Sinister Strike"].id
local SliceAndDiceId = spells["Slice and Dice"].id
local TricksId = spells["Tricks of the Trade"].id
local VanishId = spells["Vanish"].id

local actions  = {
    prep = {
        id = PreparationId,
        GetCD = function(cur_id)
            if IsSpellKnown(PreparationId) then
                if cur_id ~= PreparationId 
                   and (GetCooldown(VanishId) > 60 or cur_id == VanishId)
                then
                    return GetCooldown(PreparationId)
                end
                return 100
            end
            return 1000
        end,
        info = spells["Preparation"].name,
    },
    vanish = {
        id = VanishId,
        GetCD = function(cur_id, combo_points, anticipation_stacks)
            if not (not IsStealthed() or cur_id == AmbushId)
               and ((anticipation_stacks 
                     and anticipation_stacks<4)
                    or (combo_points < 4
                        and (GetTimeLeft("player", DeepInsight, "HELPFUL"))>0))
            then
                --pool energy if no shadow focus
                local has_shadow_focus = (IsSpellKnown(ShadowFocusId))
                if has_shadow_focus and UnitPower("player") > 20 then
                    return 100
                end
                local yellow = ((UnitPower("player") < 61)
                                and not has_shadow_focus)
                local vanishcd = ((cur_id == PreparationId) and 0)
                                  or GetCooldown(VanishId)
                return max(vanishcd,
                           (GetTimeLeft("player", ShadowBlades, "HELPFUL|PLAYER"))),
                       yellow
            end
            return 100
        end,
        ignoreTimeIncrement = true,
        info = spells["Vanish"].name,
    },
    ambush = {
        id = AmbushId,
        GetCD = function(cur_id, combo_points)
            if cur_id ~= AmbushId and IsStealthed() then
                return GetCooldown(AmbushId)
            end
            return 100
        end,
        build = 2,
        info = spells["Ambush"].name,
    },
    snd = {
        id = SliceAndDiceId,
        GetCD = function(cur_id, combo_points)
            if cur_id ~= SliceAndDiceId 
               and combo_points > 0
            then
                return max(GetCooldown(SliceAndDiceId),
                           (GetTimeLeft("player", SliceAndDice, "HELPFUL|PLAYER"))-2)
            end
            return 100
        end,
        isFinisher = true,
        info = spells["Slice and Dice"].name,
    },
    expose = {
        id = ExposeId,
        GetCD = function(cur_id, combo_points, anticipation_stacks)
            if combo_points < 5
               or (anticipation_stacks and anticipation_stacks < 5)
            then
                local time_left, count 
                             = GetTimeLeft("target", WeakenedArmor, "HARMFUL")
                if not count then count = 0 end
                if cur_id == ExposeId then count = count + 1 end
                if count < 3 or (time_left < 3 and cur_id ~= ExposeId) then
                    return GetCooldown(ExposeId)
                end
            end
            return 100
        end,
        build = 1,
        info = spells["Expose Armor"].name,
    },
    sb = {
        id = ShadowBladesId,
        GetCD = function(cur_id)
            if cur_id ~= ShadowBladesId then
                return GetCooldown(ShadowBladesId)
            end
            return 100
        end,
        ignoreTimeIncrement = true,
        info = spells["Shadow Blades"].name,
    },    
    ks = {
        id = KillingSpreeId,
        GetCD = function(cur_id)
            if cur_id ~= KillingSpreeId
               and UnitPower("player") < MAX_NRG_SPECIALS
               and (GetTimeLeft("player", SliceAndDice, "HELPFUL")) > 4
               and (GetTimeLeft("player", AdrenalineRush, "HELPFUL")) == 0
               and (GetTimeLeft("player", FuryoftheDestroyer, "HELPFUL")) == 0
            then
                return GetCooldown(KillingSpreeId)
            end
            return 100
        end,
        info = spells["Killing Spree"].name,
    },
    ar = {
        id = AdrenalineRushId,
        GetCD = function(cur_id)
            if cur_id ~= AdrenalineRushId
               and (UnitPower("player") < MAX_NRG_SPECIALS
                    or (GetTimeLeft("player", ShadowBlades, "HELPFUL")) > 0
                    or cur_id == ShadowBladesId)
            then
                return GetCooldown(AdrenalineRushId)
            end
            return 100
        end,
        ignoreTimeIncrement = true,
        info = spells["Adrenaline Rush"].name,
    },
    rupture = {
        id = RuptureId,
        GetCD = function(cur_id, combo_points)
            if cur_id ~= RuptureId
               and combo_points == 5 
               and not (UnitAura("player", BladeFlurry, nil, "HELPFUL"))
            then
               return max(GetCooldown(RuptureId),
                          (GetTimeLeft("target", Rupture, "HARMFUL|PLAYER"))-4)
            end
            return 100
        end,
        isFinisher = true,
        info = spells["Rupture"].name,
    },
    eviscerate = {
        id = EviscerateId,
        GetCD = function(_, combo_points, anticipation_stacks)
            if anticipation_stacks then
                if anticipation_stacks > 3
                   or (combo_points == 5 
                       and (GetTimeLeft("player", DeepInsight, "HELPFUL"))>0)
                then
                    return GetCooldown(EviscerateId)
                end
            elseif combo_points == 5 then
                return GetCooldown(EviscerateId)
            end
            return 100
        end,
        isFinisher = true,
        info = spells["Eviscerate"].name,
    },
    rs = {
        id = RevealingStrikeId,
        GetCD = function(cur_id)
            if cur_id ~= RevealingStrikeId
               and GetTimeLeft("target", RevealingStrike, "HARMFUL|PLAYER") < 2
            then
                return GetCooldown(RevealingStrikeId)
            end
            return 100
        end,
        build = 1,
        info = spells["Revealing Strike"].name,
    },
    tricks = {
        id = TricksId,
        GetCD = function(cur_id)
            if cur_id ~= TricksId and (UnitInRaid("player")
                                       or UnitInParty("player")) then
                return GetCooldown(TricksId)
            end
            return 100
        end,
        ignoreTimeIncrement = true,
        info = spells["Tricks of the Trade"].name,
    },
    ss = {
        id = SinisterStrikeId,
        GetCD = function()
            return GetCooldown(SinisterStrikeId)
        end,
        build = 1,
        info = spells["Sinister Strike"].name,
    },
}

----FUNCTIONS-------------------------------------------------------------------
-- The following are called once or infrequently, so speed isn't an issue.

-- called whenever module options are loaded
local GetDB
local SetDB
local GetTDB
local SetTDB
local GetPriority
local SetPriority

--initialize module.
class_module, class_module_name = clcInfo_Rogue:init_module(specname)

--called when template updates.
function class_module:OnTemplatesUpdate()
    tdb, GetTDB, SetTDB, GetPriority, SetPriority
      = clcInfo_Rogue:OnTemplatesUpdate(class_module_name, actions, tdb_default,
                                          specname)
end

-- called each time clcInfo initializes
function class_module.OnInitialize()
    db, GetDB, SetDB = clcInfo_Rogue:OnInitialize(class_module_name, db_default)
end

-- called when module options are opened
local function LoadCombatModule()
    local _, specialization_name = GetSpecializationInfo(2)
    clcInfo_Rogue.LoadModule(class_module_name, specialization_name, sample_prio,
                               actions, GetDB, SetDB, GetTDB, SetTDB,
                               GetPriority, SetPriority)
end
clcInfo.optionsCMLoaders[#(clcInfo.optionsCMLoaders) + 1] = LoadCombatModule

-- giving queue as command line
local function CmdCombat(args)
    clcInfo_Rogue:CmdLine(tdb, args, actions, specname)
end

clcInfo.cmdList[prio_arg] = CmdCombat

----ABILITY ICONS--------------------------------------------------------------
----Add these to the clcInfo.env (so they live with IconAura, etc.
local emod = clcInfo.env
function emod.IconCombatCurrent(...)
    return IconCurrent(actions, specname, tdb.rangePerSkill)
end
function emod.IconCombatNext(...)
    return IconNext()
end
