﻿------------------------------------------------------------------------------
-- $Id$
------------------------------------------------------------------------------
--                            module_subtlety.lua
--
--    Copyright (C) 2012 Deronda
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU Lesser General Public License as published
--  by the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--  This file is for use with clcInfo by abija.  Some of the code was pulled
--  from existing clcInfo modules and the example module.
--
--  abija has distributed clcInfo All Rights Reserved.  For more information
--  go to: http://wow.curseforge.com/addons/clctracker/
--
------------------------------------------------------------------------------
-- Author:
--      Deronda (@ Stormreaver)
------------------------------------------------------------------------------
-- Description:
--   Actions module for subtlety rogues
------------------------------------------------------------------------------
if not clcInfo then return end       -- need clcInfo to run
if not clcInfo_Rogue then return end -- need clcInfo_Rogue
local _, englishClass = UnitClass("player") -- need to be a rogue
if englishClass ~= clcInfo_Rogue.CLASS then return end

-- Get local copies of functions/tables.  This will speed up execution.
local clcInfo_Rogue = clcInfo_Rogue
local spells = clcInfo_Rogue.spells
local GetTimeLeft = clcInfo_Rogue.GetTimeLeft
local GetCooldown = clcInfo_Rogue.GetCooldown
local IconNext = clcInfo_Rogue.IconNext
local IconCurrent = clcInfo_Rogue.IconCurrent
local BleedDebuffLeftOnTarget = clcInfo_Rogue.BleedDebuffLeftOnTarget
local IsSpellKnown = IsSpellKnown
local IsStealthed = clcInfo_Rogue.IsStealthed
local UnitPower = UnitPower
local UnitAura = UnitAura
local GetNumPartyMembers = GetNumPartyMembers
local GetNumRaidMembers = GetNumRaidMembers

-- Assign module variables
local specname = "subtlety"
local prio_arg = "subtletyprio"
local sample_prio = "prep sb sd vanish premed ambush expose snd rupture eviscerate hemo backstab tricks"
local priority = {"none","none","sd","none","premed","ambush","none","snd","rupture",
                  "eviscerate","hemo","backstab","none"}
local class_module
local class_module_name
local has_hemo_glyph

local db_default = {
    char_var = false,
}
local db

local tdb_default = {
    version = clcInfo_Rogue.VERSION,
    priority = priority,
    rangePerSkill = false,
}
local tdb

-- actions --------------------------------------------------------------------
-- local copies of spell names and ids to ugly up code but speed action
-- functions up
-- we want the action functions to run as quickly as possible
local Ambush = spells["Ambush"].name
local FindWeakness = spells["Find Weakness"].name
local Hemorrhage = spells["Hemorrhage"].name
local MasterOfSubtlety = spells["Master of Subtlety"].name
local Premeditation = spells["Premeditation"].name
local Recuperate = spells["Recuperate"].name
local Rupture = spells["Rupture"].name
local ShadowDance = spells["Shadow Dance"].name
local SliceAndDice = spells["Slice and Dice"].name
local Stealth = spells["Stealth"].name
local Vanish = spells["Vanish"].name
local WeakenedArmor = spells["Weakened Armor"].name

local AmbushId = spells["Ambush"].id
local BackstabId = spells["Backstab"].id
local EviscerateId = spells["Eviscerate"].id
local ExposeId = spells["Expose Armor"].id
local Hemorrhageid = spells["Hemorrhage"].id
local PremeditationId = spells["Premeditation"].id
local PreparationId = spells["Preparation"].id
local RecuperateId = spells["Recuperate"].id
local RuptureId = spells["Rupture"].id
local ShadowDanceId = spells["Shadow Dance"].id
local ShadowFocusId = spells["Shadow Focus"].id
local SliceAndDiceId = spells["Slice and Dice"].id
local TricksId = spells["Tricks of the Trade"].id
local VanishId = spells["Vanish"].id

local actions  = {
    prep = {
        id = PreparationId,
        GetCD = function(cur_id)
            if IsSpellKnown(PreparationId) then
                if cur_id ~= PreparationId 
                   and (GetCooldown(VanishId) > 60 or cur_id == VanishId)
                then
                    return GetCooldown(PreparationId)
                end
                return 100
            end
            return 1000
        end,
        info = spells["Preparation"].name,
    },
    expose = {
        id = ExposeId,
        GetCD = function(cur_id, combo_points, anticipation_stacks)
            if combo_points < 5
               or (anticipation_stacks and anticipation_stacks < 5)
            then
                local time_left, count 
                             = GetTimeLeft("target", WeakenedArmor, "HARMFUL")
                if not count then count = 0 end
                if cur_id == ExposeId then count = count + 1 end
                if count < 3 or (time_left < 3 and cur_id ~= ExposeId) then
                    return GetCooldown(ExposeId)
                end
            end
            return 100
        end,
        build = 1,
        info = spells["Expose Armor"].name,
    },
    sb = {
        id = ShadowBladesId,
        GetCD = function(cur_id)
            if cur_id ~= ShadowBladesId then
                return GetCooldown(ShadowBladesId)
            end
            return 100
        end,
        ignoreTimeIncrement = true,
        info = spells["Shadow Blades"].name,
    },
    sd = {
        id = ShadowDanceId,
        GetCD = function(cur_id)
            if not IsStealthed() 
               and not (cur_id == AmbushId
                        or (GetTimeLeft("player", FindWeakness, "HELPFUL"))>0)
            then
                local yellow = (UnitPower("player") < 76) --pool energy
                return GetCooldown(ShadowDanceId), yellow
            end
            return 100
        end,
        ignoreTimeIncrement = true,
        info = spells["Shadow Dance"].name,
    },
    vanish = {
        id = VanishId,
        GetCD = function(cur_id, combo_points)
            local energy = UnitPower("player")
            local energy_max = (IsSpellKnown(ShadowFocusId) and 76) or 100
            local energy_min = (IsSpellKnown(ShadowFocusId) and 46) or 61
            if combo_points < 4
               and energy < energy_max
               and (not IsStealthed() or cur_id == AmbushId)
               and (GetTimeLeft("player", MasterOfSubtlety, "HELPFUL")) == 0
               and (GetTimeLeft("player", FindWeakness, "HELPFUL")) == 0
            then
                local yellow = (energy < energy_min) --pool energy
                local vanishcd = ((cur_id == PreparationId) and 0)
                                  or GetCooldown(VanishId)
                return vanishcd, yellow
            end
            return 100
        end,
        ignoreTimeIncrement = true,
        info = spells["Vanish"].name,
    },
    premed = {
        id = PremeditationId,
        GetCD = function(cur_id, combo_points)
            if cur_id ~= PremeditationId
               and combo_points < 4
               and IsStealthed()
            then
                return GetCooldown(PremeditationId)
            end
            return 100
        end,
        build = 2,
        ignoreTimeIncrement = true,
        info = spells["Premeditation"].name,
    },
    ambush = {
        id = AmbushId,
        GetCD = function(cur_id, combo_points, anticipation_stacks)
            local shadow_dance_left = (GetTimeLeft("player", ShadowDance, "HELPFUL"))
            if IsStealthed()
               and not (cur_id == AmbushId 
                        and shadow_dance_left == 0)
            then
                if not anticipation_stacks then
                    if combo_points < 5 then
                        return GetCooldown(AmbushId)
                    end
                elseif combo_points < 6 and anticipation_stacks == 0 then
                    return GetCooldown(AmbushId)
                elseif(anticipation_stacks < 3) then
                    return max(GetCooldown(AmbushId),
                               shadow_dance_left-2)
                end
            end
            return 100
        end,
        build = 2,
        info = spells["Ambush"].name,
    },
    snd = {
        id = SliceAndDiceId,
        GetCD = function(cur_id, combo_points)
            if cur_id ~= SliceAndDiceId and combo_points == 5 then
                return max(GetCooldown(SliceAndDiceId),
                           (GetTimeLeft("player", SliceAndDice, "HELPFUL"))-3)
            end
            return 100
        end,
        isFinisher = true,
        info = spells["Slice and Dice"].name,
    },
    rupture = {
        id = RuptureId,
        GetCD = function(cur_id, combo_points)
            if cur_id ~= RuptureId and combo_points == 5 then
               return max(GetCooldown(RuptureId),
                          (GetTimeLeft("target", Rupture, "HARMFUL|PLAYER"))-4)
            end
            return 100
        end,
        isFinisher = true,
        info = spells["Rupture"].name,
    },
    eviscerate = {
        id = EviscerateId,
        GetCD = function(_, combo_points)
            if combo_points == 5 then
                return GetCooldown(EviscerateId)
            end
            return 100
        end,
        isFinisher = true,
        info = spells["Eviscerate"].name,
    },
    hemo = {
        id = Hemorrhageid,
        GetCD = function(cur_id, combo_points)
            if cur_id ~= Hemorrhageid
               and not IsStealthed()
               and (combo_points < 4 
                    or (combo_points < 5 and UnitPower("player") > 80)) then
                return max(GetCooldown(Hemorrhageid),
                           (GetTimeLeft("target", Hemorrhage, "HARMFUL|PLAYER")))
            end
            return 100
        end,
        build = 1,
        info = spells["Hemorrhage"].name,
    },     
    backstab = {
        id = BackstabId,
        GetCD = function(_, combo_points)
            if not IsStealthed() then
                if combo_points < 4 and GetCooldown(ShadowDanceId) > 7 then
                    return GetCooldown(BackstabId)
                elseif combo_points < 5 and GetCooldown(ShadowDanceId)>=2 then
                    return GetCooldown(BackstabId)
                end
            end
            return 100
        end,
        build = 1,
        info = spells["Backstab"].name,
    },  
    tricks = {
        id = TricksId,
        GetCD = function(cur_id)
            if cur_id ~= TricksId and (UnitInRaid("player")
                                       or UnitInParty("player")) then
                return GetCooldown(TricksId)
            end
            return 100
        end,
        ignoreTimeIncrement = true,
        info = spells["Tricks of the Trade"].name,
    },
}

----FUNCTIONS-------------------------------------------------------------------
-- The following are called once or infrequently, so speed isn't an issue.

-- called whenever module options are loaded
local GetDB
local SetDB
local GetTDB
local SetTDB
local GetPriority
local SetPriority

--initialize module.
class_module, class_module_name = clcInfo_Rogue:init_module(specname)

--called when template updates.
function class_module:OnTemplatesUpdate()
    tdb, GetTDB, SetTDB, GetPriority, SetPriority 
        = clcInfo_Rogue:OnTemplatesUpdate(class_module_name, actions, 
                                          tdb_default, specname)
end

-- called each time clcInfo initializes
function class_module.OnInitialize()
    db, GetDB, SetDB = clcInfo_Rogue:OnInitialize(class_module_name, db_default)
end

-- called when module options are opened
local function LoadSubtletyModule()
    local _, specialization_name = GetSpecializationInfo(3)
    clcInfo_Rogue.LoadModule(class_module_name, specialization_name, sample_prio,
                               actions, GetDB, SetDB, GetTDB, SetTDB,
                               GetPriority, SetPriority)
end
clcInfo.optionsCMLoaders[#(clcInfo.optionsCMLoaders) + 1] = LoadSubtletyModule

-- giving queue as command line
local function CmdSubtlety(args)
    clcInfo_Rogue:CmdLine(tdb, args, actions, specname)
end

clcInfo.cmdList[prio_arg] = CmdSubtlety

----ABILITY ICONS--------------------------------------------------------------
----Add these to the clcInfo.env (so they live with IconAura, etc.
local emod = clcInfo.env
function emod.IconSubtletyCurrent(...)
    return IconCurrent(actions, specname, tdb.rangePerSkill)
end
function emod.IconSubtletyNext(...)
    return IconNext()
end
